package eu.ztsh.garmin.mapbox

import android.content.res.Resources
import android.util.Log
import com.mapbox.maps.EdgeInsets
import com.mapbox.maps.MapView
import com.mapbox.maps.Style
import com.mapbox.maps.plugin.gestures.gestures
import com.mapbox.maps.plugin.locationcomponent.location
import com.mapbox.maps.plugin.viewport.data.FollowPuckViewportStateOptions
import com.mapbox.maps.plugin.viewport.state.FollowPuckViewportState
import com.mapbox.maps.plugin.viewport.viewport
import eu.ztsh.garmin.MainActivity

class MapControl(private val context: MainActivity, private val mapView: MapView, private val resources: Resources) {

    fun init() {
        mapView.mapboxMap.loadStyle(Style.TRAFFIC_DAY) // TODO: base on sun position
        follow(true)
        setGestures(mapView)
    }

    fun follow(immediately: Boolean = false) {
        mapView.viewport.apply {
            // transition to followPuckViewportState with default transition
            val followPuckViewportState: FollowPuckViewportState = this.makeFollowPuckViewportState(
                FollowPuckViewportStateOptions.Builder()
//                .bearing(FollowPuckViewportStateBearing.Constant(0.0))
                    .padding(EdgeInsets(200.0 * resources.displayMetrics.density, 0.0, 0.0, 0.0))
                    .build()
            )
            if (immediately) {
                val immediateTransition = this.makeImmediateViewportTransition()
                this.transitionTo(followPuckViewportState, immediateTransition) { success ->
                    Log.d(TAG, "follow: $success")
                }
            } else {
                this.transitionTo(followPuckViewportState) { success ->
                    Log.d(TAG, "follow: $success")
                }
            }
        }
    }

    private fun setGestures(mapView: MapView) {
        mapView.gestures.apply {
            addOnMapClickListener { point ->
                mapView.location.isLocatedAt(point) { isPuckLocatedAtPoint ->
                    if (isPuckLocatedAtPoint) {
                        follow()
                    }
                }
                true
            }
        }
    }

    companion object {

        const val TAG = "MAPCTRL"

    }
}